@@ -1,25 +1,23 @@
 package Giddy;
 
-BEGIN {
-	use version 0.77; our $VERSION = version->declare("v0.11.0");
-}
-
 # ABSTRACT: Schema-less, versioned media/document database based on Git.
 
 use Any::Moose;
 use namespace::autoclean;
 
 use Carp;
-use File::Spec;
 use Giddy::Database;
 
+our $VERSION = "0.012_001";
+$VERSION = eval $VERSION;
+
 =head1 NAME
 
 Giddy - Schema-less, versioned media/document database based on Git.
 
 =head1 VERSION
 
-version v0.11.0
+version 0.012_001
 
 =head1 SYNOPSIS
 
@@ -27,17 +25,18 @@ version v0.11.0
 
 	my $giddy = Giddy->new;
 
-	my $db = $giddy->get_database('/path/to/database');
+	my $db = $giddy->getdb('/path/to/database');
 
 =head1 DESCRIPTION
 
 WARNING: THIS IS ALPHA SOFTWARE, RELEASED FOR TESTING PURPOSES ONLY. DO
-NOT USE IT ON A PRODUCTION ENVIRONMENT YET. IT'S INCOMPLETE, BUG-RIDDEN,
-AND WILL RUN OVER YOUR CAT.
+NOT USE IT ON A PRODUCTION ENVIRONMENT YET. IT'S INCOMPLETE, BUG-RIDDEN, LIKELY
+TO CHANGE, AND WILL RUN OVER YOUR CAT.
 
-Giddy is a schema-less (as in NoSQL), versioned database system built on
-top of Git. A database in Giddy is simply a Git repository, providing the
-database with automatic, comprehensive versioning and distributive capabilities.
+Giddy is a schema-less (as in NoSQL), versioned database system for Unix-like
+operating systems, built on top of Git. A database in Giddy is simply a Git
+repository, providing the database with automatic, comprehensive versioning and
+distributive capabilities.
 
 As opposed to most modern database systems, Giddy aims to be human editable.
 One can create/edit/delete database entries with nothing but a text editor
@@ -83,7 +82,9 @@ Returns a L<Giddy::Database> object tied to a Git repository located on the file
 system. Please provide full path names to prevent potential problems.
 
 If the path doesn't exist, Giddy will attempt to create it and initialize it
-as a Git repository.
+as a Git repository. It will also create an empty file called ".giddy" inside
+the database and perform an initial commit. You can safely remove that file
+after that.
 
 =cut
 
@@ -97,12 +98,19 @@ sub get_database {
 		unless $path;
 
 	# is this an existing database or a new one?
-	if (-d $path && -d File::Spec->catdir($path, '.git')) {
+	if (-d $path && -d "${path}/.git") {
 		# existing
 		return Giddy::Database->new(_repo => Git::Repository->new(work_tree => $path));
 	} else {
 		# new one
-		return Giddy::Database->new(_repo => Git::Repository->create(init => $path));
+		my $db = Giddy::Database->new(_repo => Git::Repository->create(init => $path));
+		
+		# create an empty .giddy file, stage it and commit, so our database will be "live"
+		$db->_touch('.giddy');
+		$db->stage('.giddy');
+		$db->commit('initial commit');
+
+		return $db;
 	}
 }
 
